/* ************************************************************************
*   File: conv-c.c                                  NOT PART OF CIRCLEMUD
*
*  Usage: Converts Easy Color "2.2" codes in C FILES to CircColor codes.
*
*  QUICK 'N' DIRTY 
*  Based on Trevor Man's modifications to my original Easy Color          *
*  Modifications Copyright Trevor Man 1997                                *
*  Based on the Easy Color patch by mud@proimages.proimages.com           *
*  mud@proimages.com is mc2@geocities.com (new address)                   *
*                                                                         *
*  This will convert Easy Color codes to the same codes used by CircEdit  *
*  and CircColor.  
*  Simply compile this then do "conv-c < infile > outfile"
*
*  gcc -o conv-c  conv-c.c
*
*  Also see "conv-txt.c" which will convert color codes in your non-C files.
*  This file converts codes in C source files (the difference is
*  that in .c files, the codes appear as \\c##, not \c##).
*
*  If your files are >100k, change FILESIZE to the appropriate number of
*  bytes.
*
*  CircEdit and CircColor (C) 1997 M.C. Lewis 
************************************************************************ */
#define FILESIZE 100000

#include <stdio.h>
#include <stdlib.h>

#define CNRM  "/c0"
#define CBLK  "/cl"
#define CRED  "/cr"
#define CGRN  "/cg"
#define CYEL  "/cy"
#define CBLU  "/cb"
#define CMAG  "/cm"
#define CCYN  "/cc"
#define CWHT  "/cw"
#define CNUL  ""

#define BBLK  "/cL"
#define BRED  "/cR"
#define BGRN  "/cG"
#define BYEL  "/cY"
#define BBLU  "/cB"
#define BMAG  "/cM"
#define BCYN  "/cC"
#define BWHT  "/cW"

#define BKBLK  "/bl"
#define BKRED  "/br"
#define BKGRN  "/bg"
#define BKYEL  "/by"
#define BKBLU  "/bb"
#define BKMAG  "/bm"
#define BKCYN  "/bc"
#define BKWHT  "/bw"

#define CAMP  "&"
#define CSLH  "\\"

#define CUDL  "/cu"	/* Underline ANSI code */
#define CFSH  "/cf"	/* Flashing ANSI code.  Change to #define CFSH "" if
                         * you want to disable flashing colour codes
                         */
#define CRVS  "/cv" /* Reverse video ANSI code */

const char *COLOURLIST[] = {CNRM, CRED, CGRN, CYEL, CBLU, CMAG, CCYN, CWHT,
			    BRED, BGRN, BYEL, BBLU, BMAG, BCYN, BWHT,
			    BKRED, BKGRN, BKYEL, BKBLU, BKMAG, BKCYN, BKWHT,
			    CAMP, CSLH, BKBLK, CBLK, CFSH, CRVS, CUDL, BBLK };

#define MAX_COLORS 30

int isnum(char s)
{
  return( (s>='0') && (s<='9') );
}

int is_colour(char code)
{
  switch (code) {
  /* Normal colours */
  case  'k': return 25; break;	/* Black */
  case  'r': return 1;	break;	/* Red */
  case  'g': return 2;	break;	/* Green */
  case  'y': return 3;	break;	/* Yellow */
  case  'b': return 4;	break;	/* Blue */
  case  'm': return 5;	break;	/* Magenta */
  case  'c': return 6;	break;	/* Cyan */
  case  'w': return 7;	break;	/* White */

  /* Bold colours */
  case  'K': return 29; break;  /* Bold black (Just for completeness) */
  case  'R': return 8;	break;	/* Bold red */
  case  'G': return 9;	break;	/* Bold green */
  case  'Y': return 10;	break;	/* Bold yellow */
  case  'B': return 11;	break;	/* Bold blue */
  case  'M': return 12;	break;	/* Bold magenta */
  case  'C': return 13;	break;	/* Bold cyan */
  case  'W': return 14;	break;	/* Bold white */
  
  /* Background colours */
  case  '0': return 24;	break; 	/* Black background */
  case  '1': return 15;	break;	/* Red background */
  case  '2': return 16;	break;	/* Green background */
  case  '3': return 17;	break;	/* Yellow background */
  case  '4': return 18;	break;	/* Blue background */
  case  '5': return 19;	break;	/* Magenta background */
  case  '6': return 20;	break;	/* Cyan background */
  case  '7': return 21;	break;	/* White background */

  /* Misc characters */
  case  '&': return 22;	break;	/* The & character */
  case '\\': return 23;	break;	/* The \ character */
  
  /* Special codes */
  case  'n': return 0;	break;	/* Normal */
  case  'f': return 26;	break;	/* Flash */
  case  'v': return 27; break;	/* Reverse video */
  case  'u': return 28; break;	/* Underline (Only for mono screens) */

  default:   return -1;	break;
  }
  return -1;
}

void proc_color(char *inbuf, int colour)
{
  register int j = 0, p = 0;
  int k, max, c = 0;
  char out_buf[FILESIZE];

  if (inbuf[0] == '\0')
    return;

  while (inbuf[j] != '\0') {
    if ((inbuf[j]=='\\') && (inbuf[j+1]=='c')
        && isnum(inbuf[j + 2]) && isnum(inbuf[j + 3])) {
      c = (inbuf[j + 2] - '0')*10 + inbuf[j + 3]-'0';
      j += 4;
    } else if ((inbuf[j] == '&') && !(is_colour(inbuf[j + 1]) == -1)) {
      c = is_colour(inbuf[j + 1]);
      j += 2;
    } else {
      out_buf[p] = inbuf[j];
      j++;
      p++;
      continue;
    }
    if (c > MAX_COLORS)
      c = 0;
    max = strlen(COLOURLIST[c]);
    if (colour || max == 1)
      for (k = 0; k < max; k++) {
        out_buf[p] = COLOURLIST[c][k];
	p++;
      }
  }

  out_buf[p] = '\0';

  strcpy(inbuf, out_buf);
}



void main(void)
{
	char buf[FILESIZE];
	int pos = 0, insq=0, indq=0, inquot=0, incomment=0;
	while (!feof(stdin))
	{	
		buf[pos] = fgetc(stdin);


if((pos>1)&&!incomment&&(buf[pos]=='\'')&&!indq)
		{
		if((buf[pos-1]=='\\')&&(buf[pos-2]=='\\'))
			insq = !insq;
		else
			if(buf[pos-1]!='\\')
				insq = !insq;
		
		/*if(insq) buf[++pos]='S';
		else  buf[++pos]='<';*/
		} 

if((pos>1)&&!incomment&&(buf[pos]=='\"')&&!insq)
		{
		if((buf[pos-1]=='\\')&&(buf[pos-2]=='\\'))
			indq = !indq;
		else
			if(buf[pos-1]!='\\')
				indq = !indq;
		
		/*if(indq) buf[++pos]='D';
		else  buf[++pos]='E';*/
		} 




inquot = (insq||indq);


if((pos>1)&&!incomment&&((buf[pos]=='\'')||(buf[pos]=='\"')))
		{
		if((buf[pos-1]=='\\')&&(buf[pos-2]=='\\'))
			inquot = !inquot;
		else
			if(buf[pos-1]!='\\')
				inquot = !inquot;
		
		/*if(inquot) buf[++pos]='Q';
		else  buf[++pos]='E';*/
		} 

		if((pos>0)&&!inquot&&!incomment)
		{
		if(buf[pos]=='*') /* are we in comment? */
		 if(buf[pos-1]=='/') incomment = 1;
		}

		
		if((inquot)&&(pos>1)) 
		{
		if(buf[pos]=='c')
		 if(buf[pos-1]=='\\')
			if(buf[pos-2]=='\\')
			{
			buf[pos-1] = 'c';
			pos--;
			}
		}
		
		if((!inquot)&&(buf[pos]=='&'))
		{pos++;	buf[pos] = '&';  } /* double it
*/

		if(pos>2) /* are we out of comment? */
		 if(incomment)
		{
		if( (buf[pos]=='/') && (buf[pos-1]=='*') &&
			(buf[pos-2] != '/') )
			incomment = 0;
		}

		pos++;
			
	}
	buf[pos] = '\0';

	proc_color(buf,1);

	fwrite(buf,strlen(buf)-1,1,stdout);
}

